/*
 * DO NOT COMMIT CHANGES TO THIS FILE WITHOUT CONSULTING EITHER
 * meder@mcs.anl.gov or welch@mcs.anl.gov
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
#include <pwd.h>
#include <signal.h>


#define program_to_run "${absolute.ogsa.root}/bin/launch_uhe.sh"
#define SAFE_LEN  1024      /* max argv length */

#define is_a_shell_meta_char(c) \
             ((c) == '!'  || (c) == '$'  || (c) == '^'  || (c) == '&' || \
              (c) == '*'  || (c) == '('  || (c) == ')'  || (c) == '~' || \
              (c) == '['  || (c) == ']'  || (c) == '|'  || (c) == '{' || \
              (c) == '}'  || (c) == '\'' || (c) == '\\' || (c) == '"' || \
              (c) == ';'  || (c) == '<'  || (c) == '>'  || (c) == '?')

int string_is_not_safe(char * arg)
{
    char * ptr;
    int    x;

    for (ptr = arg, x=1; *ptr != '\0'; ptr++, x++)
    {
        if (is_a_shell_meta_char(*ptr))
        {
           fprintf(stderr, "char %d in string %s\n", x, arg);
           return(1);
        }
    }
    return(0);
}

char * safe_dup(char * arg)
{
    char *dupped_arg;

    dupped_arg = (char *) malloc(SAFE_LEN);
    /* check for malloc failure */
    if (dupped_arg == NULL)
    {
       fprintf(stderr, "Error: malloc for dup of argument string failed\n");
       exit(8);
    }

    strncpy(dupped_arg, arg, SAFE_LEN);
    dupped_arg[SAFE_LEN - 1] = '\0';
    if (strlen(dupped_arg) == SAFE_LEN - 1)
    {
       fprintf(stderr, "Error: argument string length too long\n");
       exit(9);
    } 
    if (string_is_not_safe(dupped_arg))
    {
       fprintf(stderr, "error: unsafe char found.\n");
       exit(10);
    }
    return dupped_arg;
}

/*
 *  parameters
 *  ----------
 *  0:  user cert subject
 */
int
main(
    int                                     argc,
    char *                                  argv[])
{
    char *                                  user_name;
    char *                                  user_dir;
    char * 			            master_dn;
    char *                                  program;
    int                                     rc;
    int                                     pid;
    char *                                  port;
    char *                                  master_host;
    char *                                  master_port;
    struct passwd *                         pw_ent;
    char *                                  arg[8];
    char *                                  env[10];
    sigset_t                                sigset;
    sigset_t                                sigsave;
    int                                     i;
    
    
    /*
     * initialize sigset to block all signals
     */
    if(sigfillset(&sigset)|| sigprocmask(SIG_BLOCK,&sigset,&sigsave))
    {
        /* log to stderr since we do not know the intended log file yet */
        fprintf(stderr, "Failed to block signals, reason: %s\n",
                strerror(errno));
        exit(1);        
    }

    program = safe_dup(argv[0]);
    
    if(argc != 6)
    {
        fprintf(stderr, "wrong number of arguments\n");
        fprintf(stderr, "Usage: %s <user name> <port> <master dn> <masterHost> <masterPort>\n", program);
        exit(1);
    }
    if (( strcmp(argv[1], "-help") == 0) ||
        ( strcmp(argv[1], "-usage") == 0))
    {
        fprintf(stderr, "Usage: %s <user name> <port> <master dn> <masterHost> <masterPort>\n", program);
        exit(0);
    }

    if ( strcmp(argv[1], "root") == 0) 
    {
        fprintf(stderr, "Error: setuid root is not allowed\n");
        exit(2);
    }

    user_name   = safe_dup(argv[1]);
    port        = safe_dup(argv[2]);
    master_dn   = safe_dup(argv[3]);
    master_host  = safe_dup(argv[4]);
    master_port  = safe_dup(argv[5]);

    if ( atoi(port) < 1 )
    {
        fprintf(stderr, "Error: invalid port\n");
        exit(3);
    }

    if ((pw_ent = getpwnam(user_name)) == NULL)
    {
        fprintf(stderr, "invalid user. user name %s.\n", user_name);
        exit(4);
    }

    /* if it is root or an error, then it is an error */
    if(pw_ent->pw_uid < 1)
    {
        fprintf(stderr, "invalid user name - %s.\n", user_name);
        exit(4);
    }

    rc = setgid(pw_ent->pw_gid);
    if(rc != 0)
    {
        fprintf(stderr, "could not setgid %d.\n", pw_ent->pw_gid);
        exit(8);
    }

    rc = setuid(pw_ent->pw_uid);
    if(rc != 0)
    {
        fprintf(stderr, "could not setuid %d.\n", pw_ent->pw_uid);
        exit(5);
    }

    /* need to fork to be sure that the UHE is launched asynchronously - Java
     * doesn't make any guarantees
     */
    
    pid = fork();
    fprintf(stdout, "pid is %d\n", pid);
    if(pid < 0)
    {
        fprintf(stderr, "Fork failed: %s\n", strerror(errno));
        exit(6);
    }
    if (pid != 0)
    {
        /* parent exits here */
        exit(0);
    }
    
    /* child continues here */
    setsid();

    /* restore previously saved/set signals */
    if( sigprocmask(SIG_SETMASK,&sigsave,NULL))
    {
        /* log to stderr since we do not know the intended log file yet */
        fprintf(stderr, "Failed to restore signals, reason: %s\n",
                strerror(errno));
        exit(1);        
    }

    arg[0] = program_to_run;
    arg[1] = pw_ent->pw_dir == NULL ? "/" : pw_ent->pw_dir;
    arg[2] = port;
    arg[3] = user_name;
    arg[4] = master_dn;
    arg[5] = master_host;
    arg[6] = master_port;
    arg[7] = NULL;

    i = 0;
    env[i++] = "PATH=/bin:/usr/bin";
    env[i] = malloc(strlen("USER=") + strlen(user_name) + 1);
    if(env[i] == NULL)
    {
        fprintf(stderr, "malloc failed\n");
        exit(9);        
    }
    sprintf(env[i++],"USER=%s", user_name);
    
    env[i] = malloc(strlen("LOGNAME=") + strlen(user_name) + 1);
    if(env[i] == NULL)
    {
        fprintf(stderr, "malloc failed\n");
        exit(9);        
    }
    sprintf(env[i++],"LOGNAME=%s", user_name);
    
    env[i] = malloc(strlen("LOGIN=") + strlen(user_name) + 1);
    if(env[i] == NULL)
    {
        fprintf(stderr, "malloc failed\n");
        exit(9);        
    }
    sprintf(env[i++],"LOGIN=%s", user_name);

    if(pw_ent->pw_dir)
    { 
        env[i] = malloc(strlen("HOME=") + strlen(pw_ent->pw_dir) + 1);
        if(env[i] == NULL)
        {
            fprintf(stderr, "malloc failed\n");
            exit(9);        
        }
        sprintf(env[i++],"HOME=%s", pw_ent->pw_dir);
    }

    if(pw_ent->pw_shell)
    {
        env[i] = malloc(strlen("SHELL=") + strlen(pw_ent->pw_shell) + 1);
        if(env[i] == NULL)
        {
            fprintf(stderr, "malloc failed\n");
            exit(9);        
        }
        sprintf(env[i++],"SHELL=%s", pw_ent->pw_shell);        
    }
    
    env[i] = NULL;

    if (execve(program_to_run, arg, env) != 0)
    {
        fprintf(stderr, "Exec failed: %s\n", strerror(errno));
        exit(7);
    }

    exit(0);
}
