#ifndef GLOBUS_DONT_DOCUMENT_INTERNAL
/**
 * @file globus_gsoap_error_plugin.h
 * Globus GSOAP Error Plugin
 * @author Sam Lang
 *
 * $RCSfile $
 * $Revision $
 * $Date $
 */
#endif

/**
 * @mainpage Globus gSOAP Error Plugin
 *
 * gSOAP uses an error handling scheme whereby integer error codes
 * are returned from gSOAP functions.  This does not map well
 * to the Globus Toolkit's chained model of error handling.  This
 * module provides a plugin to gSOAP which can be used to maintain
 * globus errors within gSOAP execution control.
 *
 * The plugin should first be registered with a soap structure before
 * any gSOAP calls are made:
 *
 * \verbatim
   soap_register_plugin(soap, globus_gsoap_error_plugin_register);
   \endverbatim
 *
 * For all globus calls (a <em>globus_result_t</em> is returned) where
 * exeucution is returned to a gSOAP call, the globus error should be
 * wrapped in the error plugin:
 * 
 * \verbatim
   globus_gsoap_error_plugin_set_result(soap, result);
   \endverbatim
 * 
 * When gSOAP finally returns from a call with an error (!= SOAP_OK),
 * the error in the error plugin should be checked:
 *
 * \verbatim
   globus_gsoap_error_plugin_get_result(soap, result);
   \endverbatim
 *
 *
 * @htmlonly
 * <a href="main.html" target="_top">View documentation without frames</a><br>
 * <a href="index.html" target=_top">View documentation with frames</a><br>
 * @endhtmlonly
 */

#ifndef _GLOBUS_GSOAP_ERROR_PLUGIN_H_
#define _GLOBUS_GSOAP_ERROR_PLUGIN_H_

#include <globus_common.h>

EXTERN_C_BEGIN


/**
 * @defgroup globus_gsoap_error_plugin_activation Activation, Deactivation
 *
 * The Globus gSOAP Error Plugin library uses standard 
 * Globus module activation and deactivation.
 * Before the library is used, the following function
 * must be called:
 *
 * @code
 *      globus_module_activate(GLOBUS_GSOAP_ERROR_PLUGIN_MODULE)
 * @endcode
 *
 *
 * This function returns GLOBUS_SUCCESS if the Globus gSOAP Error Plugin Module
 * was successfully
 * initialized, and you are therefore allowed to subsequently use
 * the library functions.  Otherwise, an error code is returned, and
 * library functions should not be subsequently called. This
 * function may be called multiple times.
 *
 * To deactivate the Globus gSOAP Error Plugin, the following 
 * function must be called:
 *
 * @code
 *    globus_module_deactivate(GLOBUS_GSOAP_ERROR_PLUGIN_MODULE)
 * @endcode
 *
 * This function should be called once for each time 
 * the module was activated.
 */
/** Module descriptor
 * @ingroup activation
 */
#define GLOBUS_GSOAP_ERROR_PLUGIN_MODULE (&globus_gsoap_error_plugin_module)

extern
globus_module_descriptor_t              globus_gsoap_error_plugin_module;

#define GLOBUS_GSOAP_ERROR_PLUGIN_ID "globus_gsoap_error_plugin"

/** 
 * @defgroup globus_gsoap_error_plugin_debugging Debugging
 *
 * Before the module is activated for the first time in a process, certain
 * environment variables can be set to modify some of the debugging 
 * behavior of globus_ogsi_types module:
 *
 * <b>GLOBUS_GSOAP_ERROR_PLUGIN_DEBUG_LEVEL</b>=<em>level</em>
 *
 * The variable level indicates the amount of debugging output to be
 * generated.  The value can be any of the following mutually 
 * exclusive values:
 *
 * @b INFO - <em>shows lowest priority debug statements</em><br>
 * @b DEBUG - <em>includes most debugging statements</em><br>
 * @b WARN - <em>produces error related debugging output</em><br>
 * @b FUNCTIONS - <em>shows ENTRANCE and EXIT of all functions</em><br>
 * @b ALL - <em>all of the above</em><br>
 *
 * These values can also be or-ed together:
 *
 * <em> "DEBUG|FUNCTIONS"</em>
 *
 * NOTE:  A given level doesn't necessarily produce any debugging output
 */

/** 
 * @defgroup globus_gsoap_error_plugin Routines
 *  
 * Routines defined by the Globus OGSI Types Module.
 */
typedef struct globus_gsoap_error_plugin_handle_s * 
                                        globus_gsoap_error_plugin_handle_t;

#include <stdsoap2.h>

globus_result_t
globus_gsoap_error_plugin_set_object(
    struct soap *                       soap,
    globus_object_t *                   error_object);

globus_result_t
globus_gsoap_error_plugin_set_result(
    struct soap *                       soap,
    globus_result_t                     result);

globus_result_t
globus_gsoap_error_plugin_get_result(
    struct soap *                       soap);

globus_object_t *
globus_gsoap_error_plugin_get_object(
    struct soap *                       soap);

int
globus_gsoap_error_plugin_register(
    struct soap *                       soap,
    struct soap_plugin *                p,
    void *                              arg);

EXTERN_C_END

#endif
