#ifndef GLOBUS_DONT_DOCUMENT_INTERNAL
/**
 * @file globus_ogsi_service_plugin.h
 * Globus OGSI Service Plugin
 * @author Sam Lang
 *
 * $RCSfile $
 * $Revision $
 * $Date $
 */
#endif

/**
 * @mainpage Globus OGSI Service Plugin
 *
 * The globus_ogsi_service_plugin library provides a plugin to gSOAP that
 * maintains the different service elements that get deserialized
 * during the parsing of a request/response.  This will most likely be
 * used in the case of service data, where possibly many service data
 * elements of different types are included within one element
 * (serviceDataValues).
 *
 * This plugin should be registered with the soap struct that is associated
 * with a particular service.  The args passed into the register function
 * should be a valid globus_ogsi_service_plugin_handle_attrs_t type, containing
 * the init, destroy, and deserialization routines for a service structure.
 *
 * First, the handle attributes should be initialized:
 * \verbatim
   globus_gsoap_service_plugin_handle_attr_init(&handle_attr,
                                                init_callback,
                                                destroy_callback,
                                                deserialize_callback);
   \endverbatim
 * 
 * Then, the attributes should be passed as the plugin arguments to the
 * register function:
 *
 * \verbatim                                                
   soap_register_plugin_args(soap, 
                             globus_ogsi_service_plugin_register, 
                             (void *) handle_attr);
   \endverbatim
 * Once deserialization has finished, the service data elements resulting
 * from the deserialization can be obtained with a call to:
 *
 * \verbatim
   globus_ogsi_service_plugin_get_service_elements(soap,
                                                   &mylist);
   \endverbatim
 *
 * @htmlonly
 * <a href="main.html" target="_top">View documentation without frames</a><br>
 * <a href="index.html" target="_top">View documentation with frames</a><br>
 * @endhtmlonly
 */

#ifndef _GLOBUS_OGSI_SERVICE_PLUGIN_H_
#define _GLOBUS_OGSI_SERVICE_PLUGIN_H_

#include <globus_common.h>

EXTERN_C_BEGIN

/**
 * @defgroup globus_ogsi_service_plugin_activation Activation, Deactivation
 *
 * Globus OGSI Types uses standard Globus module activation and deactivation.
 * Before any Globus OGSI functions are called, the following function
 * must be called:
 *
 * @code
 *      globus_module_activate(GLOBUS_OGSI_SERVICE_PLUGIN_MODULE)
 * @endcode
 *
 *
 * This function returns GLOBUS_SUCCESS if the module
 * was successfully
 * initialized, and you are therefore allowed to subsequently call
 * register the plugin to soap structures, and make library function calls.  
 * Otherwise, an error code is returned, and
 * module functions should not be subsequently called. This
 * function may be called multiple times.
 *
 * To deactivate the module, the following function must be called:
 *
 * @code
 *    globus_module_deactivate(GLOBUS_OGSI_SERVICE_PLUGIN_MODULE)
 * @endcode
 *
 * This function should be called once for each time 
 * the module was activated.
 */
/** Module descriptor
 * @ingroup globus_ogsi_service_plugin_activation
 */
#define GLOBUS_OGSI_SERVICE_PLUGIN_MODULE (&globus_ogsi_service_plugin_module)

extern
globus_module_descriptor_t              globus_ogsi_service_plugin_module;

#define GLOBUS_OGSI_SERVICE_PLUGIN_ID "globus_ogsi_service_plugin"

/** 
 * @defgroup globus_ogsi_service_plugin_debugging Debugging
 *
 * Before the module is activated for the first time in a process, certain
 * environment variables can be set to modify some of the debugging 
 * behavior of globus_ogsi_service_plugin module:
 *
 * <b>GLOBUS_OGSI_SERVICE_PLUGIN_DEBUG_LEVEL</b>=<em>level</em>
 *
 * The variable level indicates the amount of debugging output to be
 * generated.  The value can be any of the following mutually 
 * exclusive values:
 *
 * @b INFO - <em>shows lowest priority debug statements</em><br>
 * @b DEBUG - <em>includes most debugging statements</em><br>
 * @b WARN - <em>produces error related debugging output</em><br>
 * @b FUNCTIONS - <em>shows ENTRANCE and EXIT of all functions</em><br>
 * @b ALL - <em>all of the above</em><br>
 *
 * These values can also be or-ed together:
 *
 * <em> "DEBUG|FUNCTIONS"</em>
 *
 * NOTE:  A given level doesn't necessarily produce any debugging output
 */

/**
 * @defgroup globus_ogsi_service_plugin Routines
 *
 * Routines defined by the Globus OGSI Service Plugin Module.
 */

#include <stdsoap2.h>

typedef void * (* globus_ogsi_service_plugin_init_callback_t) (
    struct soap *                       soap,
    void **                             service_handle);

typedef void (* globus_ogsi_service_plugin_destroy_callback_t) (
    struct soap *                       soap,
    void *                              service_handle);

typedef void (* globus_ogsi_service_plugin_serialize_callback_t) (
    struct soap *                       soap,
    void *                              service_handle);

typedef int (* globus_ogsi_service_plugin_deserialize_callback_t) (
    struct soap *                       soap,
    const char *                        sde_name,
    void *                              service_handle);

typedef struct globus_ogsi_service_plugin_handle_s * 
                                      globus_ogsi_service_plugin_handle_t;

typedef struct globus_ogsi_service_plugin_handle_attr_s *
                                      globus_ogsi_service_plugin_handle_attr_t;

globus_result_t
globus_ogsi_service_plugin_handle_attr_init(
    globus_ogsi_service_plugin_handle_attr_t *        handle_attr,
    globus_ogsi_service_plugin_init_callback_t        init_callback,
    globus_ogsi_service_plugin_destroy_callback_t     destroy_callback,
    globus_ogsi_service_plugin_deserialize_callback_t deserialize_callback,
    globus_ogsi_service_plugin_serialize_callback_t   serialize_callback);

globus_result_t
globus_ogsi_service_plugin_handle_attr_destroy(
    globus_ogsi_service_plugin_handle_attr_t   handle_attr);

int 
globus_ogsi_service_plugin_register(
    struct soap *                       soap,
    struct soap_plugin *                p,
    void *                              arg);

globus_result_t
globus_ogsi_service_plugin_serialize(
    struct soap *                       soap,
    void *                              service_element);

globus_result_t
globus_ogsi_service_plugin_deserialize(
    struct soap *                       soap,
    const char *                        sde_name);

globus_result_t
globus_ogsi_service_plugin_get_service_elements(
    struct soap *                       soap,
    globus_list_t **                    service_elements);

globus_result_t
globus_ogsi_service_plugin_add_service_element(
    struct soap *                       soap,
    void *                              etype);

#endif /* _GLOBUS_OGSI_SERVICE_PLUGIN_H_ */
