#ifndef GLOBUS_DONT_DOCUMENT_INTERNAL
/**
 * @file globus_ogsi_types.h
 * Globus Core Types
 * @author Sam Lang
 *
 * $RCSfile $
 * $Revision $
 * $Date $
 */
#endif

/**
 * @mainpage Globus OGSI Types API
 *
 * The globus_ogsi_types library provides special handling for 
 * ExtensibilityType
 * elements by mapping types within an ExtensibilityType to the appropriate
 * (de)marshalling routines.  
 *
 * Before any marshalling routines are called, the globus_ogsi_types module
 * should be activated.  This initializes the map of XSD types to 
 * deserialization functions.  As types are expected to be wrapped
 * by an ExtensibilityType element, deserializers should be added to
 * the map using the \ref globus_ogsi_types_add function.  The
 * ExtensibilityType element will automatically call the deserialization
 * function (via \ref globus_ogsi_types_deserialize) as it is being
 * deserialized.
 *
 * Finally, when the application no longer needs to perform deserialization,
 * the globus_ogsi_types module should be deactivated.  This usually happens
 * at application cleanup.
 *
 * @htmlonly
 * <a href="main.html" target="_top">View documentation without frames</a><br>
 * <a href="index.html" target="_top">View documentation with frames</a><br>
 * @endhtmlonly
 */

#ifndef _GLOBUS_OGSI_TYPES_H_
#define _GLOBUS_OGSI_TYPES_H_

#include <globus_common.h>

EXTERN_C_BEGIN


/**
 * @defgroup globus_ogsi_types_activation Activation, Deactivation
 *
 * Globus OGSI Types uses standard Globus module activation and deactivation.
 * Before any Globus OGSI functions are called, the following function
 * must be called:
 *
 * @code
 *      globus_module_activate(GLOBUS_OGSI_TYPES_MODULE)
 * @endcode
 *
 *
 * This function returns GLOBUS_SUCCESS if Globus OGSI Types Module
 * was successfully
 * initialized, and you are therefore allowed to subsequently call
 * Globus OGSI Types functions.  Otherwise, an error code is returned, and
 * Globus OGSI Types functions should not be subsequently called. This
 * function may be called multiple times.
 *
 * To deactivate Globus OGSI Types, the following function must be called:
 *
 * @code
 *    globus_module_deactivate(GLOBUS_OGSI_TYPES_MODULE)
 * @endcode
 *
 * This function should be called once for each time 
 * the module was activated.
 */
/** Module descriptor
 * @ingroup globus_ogsi_types_activation
 */
#define GLOBUS_OGSI_TYPES_MODULE (&globus_ogsi_types_module)

extern
globus_module_descriptor_t              globus_ogsi_types_module;


/** 
 * @defgroup globus_ogsi_types_debugging Debugging
 *
 * Before the module is activated for the first time in a process, certain
 * environment variables can be set to modify some of the debugging 
 * behavior of globus_ogsi_types module:
 *
 * <b>GLOBUS_OGSI_TYPES_DEBUG_LEVEL</b>=<em>level</em>
 *
 * The variable level indicates the amount of debugging output to be
 * generated.  The value can be any of the following mutually 
 * exclusive values:
 *
 * @b INFO - <em>shows lowest priority debug statements</em><br>
 * @b DEBUG - <em>includes most debugging statements</em><br>
 * @b WARN - <em>produces error related debugging output</em><br>
 * @b FUNCTIONS - <em>shows ENTRANCE and EXIT of all functions</em><br>
 * @b ALL - <em>all of the above</em><br>
 *
 * These values can also be or-ed together:
 *
 * <em> "DEBUG|FUNCTIONS"</em>
 *
 * NOTE:  A given level doesn't necessarily produce any debugging output
 */

/** 
 * @defgroup globus_ogsi_types Routines
 *  
 * Routines defined by the Globus OGSI Types Module.
 */

typedef struct globus_ogsi_extensible_type_s * globus_ogsi_extensible_type_t;
typedef struct globus_ogsi_types_handle_s * globus_ogsi_types_handle_t;

typedef globus_result_t (* globus_ogsi_types_serializer_t) (
    const char *                        tag,
    void *                              value,
    char **                             buffer,
    size_t *                            buff_length,
    void *                              user_data);

typedef void (* globus_ogsi_types_free_buffer_t) (
    char *                              buffer);

typedef globus_result_t (* globus_ogsi_types_deserializer_t) (
    const char *                        buffer,
    const char *                        typename,
    void **                             value,
    void *                              user_data);

#define GLOBUS_EXTENSIBLE_TYPE_HASHKEY_NOT_FOUND 42

globus_result_t
globus_ogsi_types_handle_init(
    globus_ogsi_types_handle_t *        handle);

globus_result_t
globus_ogsi_types_handle_destroy(
    globus_ogsi_types_handle_t          handle);

globus_result_t
globus_ogsi_extensible_type_init(
    globus_ogsi_extensible_type_t *     handle,
    const char *                        typename,
    const char *                        elementname,
    globus_ogsi_types_deserializer_t    deserializer_callback,
    globus_ogsi_types_serializer_t      serializer_callback,
    globus_ogsi_types_free_buffer_t     free_buffer_callback,
    void *                              value);

globus_result_t
globus_ogsi_extensible_type_destroy(
    globus_ogsi_extensible_type_t       type_handle);

globus_result_t
globus_ogsi_types_add_type(
    const char *                        typename,
    globus_ogsi_types_deserializer_t    deserializer);

globus_result_t
globus_ogsi_types_deserialize(
    const char *                        typename,
    globus_ogsi_extensible_type_t *     result,
    char *                              buffer,
    void *                              user_data);

globus_result_t
globus_ogsi_extensible_type_serialize(
    globus_ogsi_extensible_type_t       etype,
    char **                             buff,
    size_t *                            buff_length,
    void *                              user_data);

void
globus_ogsi_extensible_type_free_buffer(
    globus_ogsi_extensible_type_t       etype,
    char *                              buff);

globus_result_t
globus_ogsi_extensible_type_get_value(
    globus_ogsi_extensible_type_t       handle,
    void **                             value);

EXTERN_C_END

#endif /* _GLOBUS_OGSI_TYPES_H_ */
