package Grid::GPT::V1::BuildFlavors;

use strict;
use Carp;

require Exporter;
use vars       qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
use Data::Dumper;
use Grid::GPT::V1::XML;
use Grid::GPT::V1::Definitions;
use Grid::GPT::V1::FlavorDefinition;
use Grid::GPT::V1::FlavorChoices;

# set the version for version checking
$VERSION     = 0.01;

@ISA         = qw(Exporter);

package Grid::GPT::V1::BuildFlavors;

sub new {
    my ($that, %args)  = @_;
    my $class = ref($that) || $that;
    my $me  = {
               choices => [],
               standard => $args{'std'},
		};
    bless $me, $class;

    my $gpath = $ENV{GPT_LOCATION};
    
    if (!defined($gpath))
      {
        $gpath = $ENV{GLOBUS_LOCATION};
      }
    my $confdir = "$gpath/etc/gpt";

    if (defined $args{'core'}) {

      my $conf_file = "$confdir/globus_flavor_labels.conf";
      $conf_file = $args{'cfg'} if defined $args{'cfg'};
      my $choicefile = new Grid::GPT::V1::XML;
      $choicefile->read($conf_file);
      my $root = $choicefile->{'roottag'};
      $me->read_xml_choices($root);

      my $flavorlist = $me->permute();
      $me->{'flavors'} = [];
      for my $f (@$flavorlist) {
        $f->{'name'} = $f->build_label() if ! defined $f->{'name'};
        $me->{$f->{'name'}} = $f;
        push @{$me->{'flavors'}}, $f->{'name'};
      }
    }

    if (defined $args{'xml'}) {
      $me->read_xml_choices($args{'xml'});
    }

    if (defined $args{'installed'}) {

      if ($args{'installdir'}) {
        $gpath = $args{'installdir'};
      } else {
        $gpath = $ENV{GLOBUS_LOCATION} 
          || die "ERROR: Please set \$GLOBUS_LOCATION\n";
      }

      opendir(CONFDIR, "$gpath/etc/globus_core") 
        || die "ERROR: $gpath/etc/globus_core cannot be accessed\n";

      my @flavorfiles = grep {m!flavor_\w+.gpt!} readdir(CONFDIR);

      closedir(CONFDIR);
      $me->{'flavors'} = [];
      for my $ff (@flavorfiles) {
        my $obj = 
          new Grid::GPT::V1::FlavorDefinition(xmlfile => 
                                          "$gpath/etc/globus_core/$ff");
        $me->{$obj->{'name'}} = $obj;
        push @{$me->{'flavors'}}, $obj->{'name'};
      }

    }
#    print Dumper $me;
    return $me;
}

sub read_xml_choices {
  my ($me, $xml) = @_;
  for my $f (@{$xml->{'contents'}}) {
    next if ref($f) ne 'HASH';
    my $choice = new Grid::GPT::V1::FlavorChoices(xml => $f);
    push @{$me->{'choices'}}, $choice;
  }
}

sub write_xml_choices {
  my ($me, $xml) = @_;
  $xml->startTag('flavors');
  $xml->characters("\n");
  for my $f (@{$me->{'choices'}}) {
    $f->write_xml(xml => $xml);
  }
  $xml->endTag('flavors');
  $xml->characters("\n");
}

sub permute {
  my ($me, %arg) = @_;
  my @flavors;
  
  my $masterc = defined $arg{'choice'} ? $arg{'choice'} : 0;
  my $choice = $me->{'choices'}->[$masterc];
  my $oldlist = $arg{'list'};
  my $newlist = [];
  
  if (! defined($oldlist)) {
    $newlist = $choice->permute(std => $me->{'standard'});
  } else {
    for my $f (@$oldlist) {
      my $locallist = $choice->permute(flavor => $f, std => $me->{'standard'});
      push @$newlist, @$locallist;
    }
  }
  return $me->permute(choice => $masterc + 1, list => $newlist) 
    if $masterc < @{$me->{'choices'}} - 1;
  return $newlist;
}

sub build_macros {
  my ($me, $macros,$noswitches)=  @_;
  for my $f (@{$me->{'flavors'}}) {
   my $fo = $me->{$f}->build_core_configure_line();
   $macros->{"$fo->{'flavor'}_CONFIGOPTS_GPTMACRO"} = $fo->{'switches'} 
     if ! defined $noswitches;
   $macros->{"$fo->{'flavor'}_ENV_GPTMACRO"} = $fo->{'env'};
  }
}

sub clone {
    my $me = shift;
    my $clone = new Grid::GPT::V1::BuildFlavors;
    replicate($me, $clone);
    return $clone;
}

sub replicate {
  my ($rold, $newclass) = @_;
  if (ref(\$rold) eq 'SCALAR') {
    return $rold;
  } elsif (ref($rold) eq 'ARRAY') {
    my @list = @$rold;
    return \@list;
  } elsif (ref($rold) eq 'HASH') {
    my $rnew = {};
    for my $e (sort keys %$rold) {
      $rnew->{$e} = replicate($rold->{$e});
    }
    return $rnew;

  } elsif (ref($rold) eq 'Grid::GPT::V1::BuildFlavors' ) {
    for my $e (sort keys %$rold) {
      $newclass->{$e} = replicate($rold->{$e});
    }
    return;
  }
}

sub open_xml {
  my $writer = new Grid::GPT::V1::XML;  
  $writer->doctype("gpt_package_metadata","package.dtd");
  $writer->startTag("flavors",
		    Format_Version =>xml_format_version());
  $writer->characters("\n");
  return $writer;
}

sub close_xml {
  my ($writer, $filename) = @_;
  $writer->endTag('flavors');
  $writer->write($filename);
}

sub AUTOLOAD {
  use vars qw($AUTOLOAD);
  my $me = shift;
  my $type = ref($me) || croak "$me is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;   # strip fully-qualified portion
  unless (exists $me->{$name} ) {
    croak "Can't access `$name' field in object of class $type";
  } 
  if (@_) {
    return $me->{$name} = shift;
  } else {
    return $me->{$name};
  } 
}

sub DESTROY {}
END { }       # module clean-up code here (global destructor)



1;
__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

Grid::GPT::V1::BuildFlavors - Perl extension for managing package version metadata

=head1 SYNOPSIS

  use Grid::GPT::V1::BuildFlavors;
  my $a_ver = new Grid::GPT::V1::BuildFlavors(type =>'aging', major => '4', minor =>'6', age =>'2');
  my $s_ver = new Grid::GPT::V1::BuildFlavors(type =>'simple', major => '3');
  my $r_ver = new Grid::GPT::V1::BuildFlavors(type =>'range', upper_major => '3', lower_major => '2');

  my $result = $a_ver->is_compatible($s_ver);

=head1 DESCRIPTION

I<Grid::GPT::V1::BuildFlavors> is used to manage package version metadata. The
version metadata is to used to describe the version of a package as
well as dependencies to packages.

=head1 BuildFlavors Metadata Types

There are several ways to express information about the compatibility
of the different versions.  This package supports the following
schemes which are called version metadata types.

=over 4

=item aging

This metadata is used to describe the version of a package. It consists of the following fields:

=over 4

=item major 

This is the main version number of the package.

=item minor

This is a number used to version bug fixes that maintain binary compatibility.

=item age

This number denotes the backward compatability of the package.  For
example a package with a major number of 5 and an age of 2 is
compatible back to version 3.

=back

=item simple

This metadata is used to describe the version requirements of a package dependency.
It consists of only a major version number.  The dependency indicates
with this metadata that it will accept a version if the major number
falls with in the age range of the package fulfilling the dependency.

=item range

This metadata is also used to describe the version requirements of a
package dependency.  The range is compared to the specific version
number of the package fulfilling the dependency.  Not that the age
criteria is not used here.  The metadata consists of the following:

=over 4

=item upper_major

The newest major version number accepted. 

=item lower_major

The oldest major version number accepted. 

=item upper_minor

The newest minor version number accepted. 

=item lower_minor

The oldest minor version number accepted. 

=back

=back

=head1 Methods

=over 4

=item new

Create a new I<Grid::GPT::V1::BuildFlavors> object.  The following named arguments are accepted

=over 4

=item type

The type of version metadata this object will contain.

=item major

The major version number (used for aging and simple types).

=item minor

The minor version number (used only for the aging type).

=item age

The age of the major version (used only for the aging type).

=item upper_major

The newest major number allowed (used only for the range type).

=item upper_minor

The newest minor number allowed (used only for the range type).

=item lower_major

The oldest major number allowed (used only for the range type).

=item lower_minor

The oldest minor number allowed (used only for the range type).


=item obj

This passes in a L<Grid::GPT::V1::XML|Grid::GPT::V1::XML> version object.  The
version metadata is extracted from the object.

=back

=item is_compatible

Determines if the aging version metadata fulfills the requirements of
another version metadata which can be either simple version metadata
or range version metadata.  Returns 1 if the requirement is fulfilled.
Returns 0 otherwise.

=item validate

Determines if the version metadata is complete.

=item write_tag(xml_obj)

Adds version contents into an L<Grid::GPT::V1::XML|Grid::GPT::V1::XML> object. 

=item create_version_list(xml_obj)

Class function which creates a list of BuildFlavors objects from a
L<Grid::GPT::V1::XML|Grid::GPT::V1::XML> object.  The function returns a
reference to the list.


=item convert_version_list2xml(version_list_reference, xml_obj)

Class function which adds the contents of all BuildFlavors objects in a
list reference to an L<Grid::GPT::V1::XML|Grid::GPT::V1::XML> object.

=back

=head1 BuildFlavorsing Examples

As an example consider the installed package foo which has a version
number of 5.3.  As was mentioned in the previous section, this
specifies a compatibility range of 2 to 5.  Now we want to install
package fum which depends on foo.  The following table shows how the
versioning works:


    BuildFlavors specification for   BuildFlavors
    Dependency foo to fum       Type           Dependency is met
    
    1                           Simple         No 
    1 to 4                      Range          No
    4 to 4                      Range          No
    3                           Simple         Yes
    3 to 6                      Range          Yes


=head1 AUTHOR

Eric Blau <eblau@ncsa.uiuc.edu> Michael Bletzinger <mbletzin@ncsa.uiuc,edu>

=head1 SEE ALSO

perl(1) GRID::GPT::V1::XML(1).

=cut
