/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ogsa.gui;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import java.awt.Component;

import java.beans.BeanInfo;
import java.beans.Beans;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;

import java.util.Calendar;
import java.util.TimeZone;


public class DynamicParameterPanel extends JPanel {
    protected static Log logger =
        LogFactory.getLog(DynamicParameterPanel.class.getName());
    String parameterName = null;
    Class parameterType = null;
    JTextField valueTextField = null;
    JLabel valueNameLabel = null;
    boolean editable = true;
    JPanel complexTypePanel = null;
    BeanInfo beanInfo = null;
    public static final String RETURN_VALUE_NAME = "Return Value";
    boolean isReturnValue = false;

    public DynamicParameterPanel(
        String parameterName,
        Class parameterType
    ) {
        this(parameterName, parameterType, true);
    }

    public DynamicParameterPanel(
        String parameterName,
        Class parameterType,
        boolean editable
    ) {
        this.parameterName = parameterName;
        this.parameterType = parameterType;
        this.editable = editable;

        if (this.parameterName.equals(RETURN_VALUE_NAME)) {
            this.isReturnValue = true;
            this.editable = false;
        }

        logger.debug("Parameter Name: " + this.parameterName);
        logger.debug("Parameter Type: " + this.parameterType);
        logger.debug("Editable: " + this.editable);

        if ((this.parameterType != null) && this.parameterType.isArray()) {
            //Add panel w/name on border
            this.complexTypePanel = new DynamicArrayPanel(this.parameterType);
            complexTypePanel.setBorder(
                BorderFactory.createTitledBorder("Array " + this.parameterName)
            );
            complexTypePanel.setLayout(
                new BoxLayout(complexTypePanel, BoxLayout.Y_AXIS)
            );
            add(complexTypePanel);
        } else if (
            (this.parameterType == null) || (this.parameterType.isPrimitive()) ||
                (isPrimitiveObject(this.parameterType))
        ) {
            //Simple name/value field
            this.valueNameLabel = new JLabel(this.parameterName);
            add(this.valueNameLabel);

            this.valueTextField = new JTextField(20);
            add(this.valueTextField);
            this.valueTextField.setEditable(this.editable);
        } else {
            //Add panel w/name on border
            this.complexTypePanel = new JPanel();
            complexTypePanel.setBorder(
                BorderFactory.createTitledBorder(this.parameterName)
            );
            complexTypePanel.setLayout(
                new BoxLayout(complexTypePanel, BoxLayout.Y_AXIS)
            );
            add(complexTypePanel);

            //Check for a default constructor
            if (!hasDefaultConstructor()) {
                complexTypePanel.add(
                    new JLabel("No Public Default Constructor")
                );
            }

            //Get the bean info for the parameter class
            try {
                this.beanInfo = Introspector.getBeanInfo(this.parameterType);
            } catch (java.beans.IntrospectionException ie) {
                logger.error("problem introspecting parameter", ie);
            }

            //Add parameter panels for each property
            addPropertyFields();
        }
    }

    private boolean hasDefaultConstructor() {
        //Check for a default constructor
        try {
            if (!this.parameterType.getName().equals("java.util.Calendar")) {
                this.parameterType.getConstructor(new Class[0]);
            }
        } catch (java.lang.NoSuchMethodException nsme) {
            return false;
        }

        return true;
    }

    private void addPropertyFields() {
        PropertyDescriptor[] propertyDescriptors =
            this.beanInfo.getPropertyDescriptors();

        for (int index = 0; index < propertyDescriptors.length; index++) {
            Method readMethod = propertyDescriptors[index].getReadMethod();
            Method writeMethod = propertyDescriptors[index].getWriteMethod();

            if ((readMethod != null) && (writeMethod != null)) {
                //Property is readable and writable
                String propertyName = propertyDescriptors[index].getName();
                Class propertyType =
                    propertyDescriptors[index].getPropertyType();
                complexTypePanel.add(
                    new DynamicParameterPanel(
                        propertyName, propertyType, this.editable
                    )
                );
            }
        }
    }

    public Class getType() {
        return this.parameterType;
    }

    public Object getValue() {
        Object parameterValue = null;

        if (logger.isDebugEnabled()) {
            logger.debug(
                "Parameter " + this.parameterName + " is of type " +
                this.parameterType.toString()
            );
        }

        if (this.parameterType.isArray()) {
            parameterValue =
                ((DynamicArrayPanel) this.complexTypePanel).getArray();
        } else if (this.parameterType.isPrimitive()) {
            logger.debug("Parameter " + this.parameterName + " is a primitive");

            String stringValue = valueTextField.getText();

            if (stringValue.equals("")) {
                return null;
            }

            parameterValue =
                getPrimitiveWrapper(this.parameterType, stringValue);
        } else {
            //Get a new instance of the parameter's class type
            if (this.parameterType.getName().equals("java.util.Calendar")) {
                //Special case for Calendar
                parameterValue = Calendar.getInstance();
            } else if (
                this.parameterType.getName().equals("java.util.TimeZone")
            ) {
                //Special case for TimeZone
                parameterValue = TimeZone.getDefault();
            } else if (isPrimitiveObject(this.parameterType)) {
                //Boolean, Integer, Character, etc...
                //Get the constructor for the primitive wrapper class
                Constructor constructor = null;

                try {
                    constructor =
                        this.parameterType.getConstructor(
                            new Class[] { String.class }
                        );
                } catch (java.lang.NoSuchMethodException nsme) {
                    logger.error(
                        "no public default constructor for parameter " +
                        this.parameterName
                    );

                    return null;
                }

                //Setup the primitive wrapper constructor arguments
                Object[] arguments = new Object[] { valueTextField.getText() };

                //Create an instance of the primitive wrapper class
                try {
                    parameterValue = constructor.newInstance(arguments);
                } catch (java.lang.InstantiationException ie) {
                    logger.error(
                        "couldn't create an object of type " +
                        this.parameterType.getName(), ie
                    );

                    return null;
                } catch (java.lang.IllegalAccessException iae) {
                    logger.error(
                        "couldn't create an object of type " +
                        this.parameterType.getName(), iae
                    );

                    return null;
                } catch (java.lang.IllegalArgumentException iarge) {
                    logger.error(
                        "couldn't create an object of type " +
                        this.parameterType.getName(), iarge
                    );

                    return null;
                } catch (java.lang.reflect.InvocationTargetException ite) {
                    logger.error(
                        "couldn't create an object of type " +
                        this.parameterType.getName(), ite
                    );

                    return null;
                }
            } else {
                //Bean instantiation
                try {
                    parameterValue =
                        Beans.instantiate(
                            this.parameterType.getClassLoader(),
                            this.parameterType.getName()
                        );
                } catch (java.io.IOException ioe) {
                    logger.error(
                        "couldn't instantiate bean type " +
                        this.parameterType.getName(), ioe
                    );
                } catch (java.lang.ClassNotFoundException cnfe) {
                    logger.error(
                        "couldn't instantiate bean type " +
                        this.parameterType.getName(), cnfe
                    );
                }

                PropertyDescriptor[] propertyDescriptors =
                    this.beanInfo.getPropertyDescriptors();

                for (int index = 0; index < propertyDescriptors.length;
                        index++
                ) {
                    Method readMethod =
                        propertyDescriptors[index].getReadMethod();
                    Method writeMethod =
                        propertyDescriptors[index].getWriteMethod();

                    if ((readMethod != null) && (writeMethod != null)) {
                        //Set the value of the parameter's property
                        DynamicParameterPanel parameterPanel =
                            getParameterPanel(
                                propertyDescriptors[index].getName()
                            );
                        Object propertyValue = parameterPanel.getValue();

                        try {
                            writeMethod.invoke(
                                parameterValue, new Object[] { propertyValue }
                            );
                        } catch (java.lang.IllegalAccessException iae) {
                            logger.error(
                                "couldn't invoke write method " +
                                this.parameterType.getName() + "." +
                                writeMethod.getName(), iae
                            );

                            return null;
                        } catch (java.lang.IllegalArgumentException iarge) {
                            logger.error(
                                "couldn't invoke write method " +
                                this.parameterType.getName() + "." +
                                writeMethod.getName(), iarge
                            );

                            return null;
                        } catch (java.lang.reflect.InvocationTargetException ite) {
                            logger.error(
                                "couldn't invoke write method " +
                                this.parameterType.getName() + "." +
                                writeMethod.getName(), ite
                            );

                            return null;
                        }
                    }
                }
            }
        }

        return parameterValue;
    }

    private DynamicParameterPanel getParameterPanel(String parameterName) {
        JPanel complexTypePanel = (JPanel) getComponents()[0];

        Component[] components = complexTypePanel.getComponents();

        for (int index = 0; index < components.length; index++) {
            DynamicParameterPanel parameterPanel = null;

            try {
                parameterPanel = (DynamicParameterPanel) components[index];

                if (parameterPanel.getParameterName().equals(parameterName)) {
                    return (DynamicParameterPanel) components[index];
                }
            } catch (ClassCastException cce) {
            }
        }

        return null;
    }

    public void setValue(Object parameterValue) {
        if (logger.isDebugEnabled()) {
            logger.debug(
                "Parameter " + this.parameterName + " is of type " +
                this.parameterType.toString()
            );
        }

        if (this.parameterType.isArray()) {
            ((DynamicArrayPanel) this.complexTypePanel).setArray(
                parameterValue
            );
        } else if (
            this.parameterType.isPrimitive() ||
                isPrimitiveObject(this.parameterType)
        ) {
            logger.debug("Parameter " + this.parameterName + " is a primitive");

            valueTextField.setText(parameterValue.toString());
        } else {
            PropertyDescriptor[] propertyDescriptors =
                this.beanInfo.getPropertyDescriptors();

            for (int index = 0; index < propertyDescriptors.length; index++) {
                Method readMethod = propertyDescriptors[index].getReadMethod();
                Method writeMethod =
                    propertyDescriptors[index].getWriteMethod();

                if ((readMethod != null) && (writeMethod != null)) {
                    //Property is readable and writable
                    String propertyName = propertyDescriptors[index].getName();
                    Class propertyType =
                        propertyDescriptors[index].getPropertyType();

                    DynamicParameterPanel parameterPanel =
                        getParameterPanel(propertyName);

                    try {
                        parameterPanel.setValue(
                            readMethod.invoke(parameterValue, new Object[0])
                        );
                    } catch (java.lang.IllegalAccessException iae) {
                        logger.error(
                            "couldn't invoke read method " + propertyType +
                            "." + readMethod.getName(), iae
                        );
                    } catch (java.lang.reflect.InvocationTargetException ite) {
                        logger.error(
                            "couldn't invoke read method " + propertyType +
                            "." + readMethod.getName(), ite
                        );
                    }
                }
            }
        }
    }

    public String getParameterName() {
        return this.parameterName;
    }

    public Class getParameterType() {
        return this.parameterType;
    }

    protected boolean isPrimitiveObject(Class type) {
        String typeName = type.getName();

        if (
            typeName.equals("java.lang.String") ||
                typeName.equals("java.lang.Boolean") ||
                typeName.equals("java.lang.Character") ||
                typeName.equals("java.lang.Double") ||
                typeName.equals("java.lang.Float") ||
                typeName.equals("java.lang.Integer") ||
                typeName.equals("java.lang.Long")
        ) {
            return true;
        }

        return false;
    }

    protected Object getPrimitiveWrapper(
        Class type,
        String stringValue
    ) {
        String typeName = type.getName();

        //Wrap primitives in their object equivalents
        Object value = null;

        if (typeName.equals("java.lang.String")) {
            value = new String(stringValue);
        } else if (typeName.equals("boolean")) {
            value = new Boolean(stringValue);
        } else if (typeName.equals("char")) {
            value = new Character(stringValue.charAt(0));
        } else if (typeName.equals("double")) {
            value = new Double(stringValue);
        } else if (typeName.equals("float")) {
            value = new Float(stringValue);
        } else if (typeName.equals("int")) {
            value = new Integer(stringValue);
        } else if (typeName.equals("long")) {
            value = new Long(stringValue);
        } else {
            value = null;
        }

        return value;
    }
}
